@@ -346,6 +346,17 @@ init_flow(ModeInfo * mi)
 
 	/* Clear the background. */
 	MI_CLEARWINDOW(mi);
+	if (!MI_IS_ICONIC(mi)) {
+		/* clip to size of screen (for broken Xservers) */
+		XRectangle cliprect[1];
+
+		cliprect[0].x = 0;
+		cliprect[0].y = 0;
+		cliprect[0].width = MI_WIDTH(mi);
+		cliprect[0].height = MI_HEIGHT(mi);
+		XSetClipRectangles(MI_DISPLAY(mi), MI_GC(mi), 0, 0,
+			cliprect, 1, Unsorted);
+	}
 
 	/* Allocate memory. */
 	if (((sp->csegs = (XSegment *) malloc(sizeof (XSegment) *
@@ -522,20 +533,20 @@ draw_flow(ModeInfo * mi)
 			double x=X(i,b)-sp->centre.x;
 			double y=Y(i,b)-sp->centre.y;
 			double z=Z(i,b)-sp->centre.z;
-			double X=M[0][0]*x + M[0][1]*y + M[0][2]*z;
-			double Y=M[1][0]*x + M[1][1]*y + M[1][2]*z;
-			double Z=M[2][0]*x + M[2][1]*y + M[2][2]*z+sp->view.height;
+			double XM=M[0][0]*x + M[0][1]*y + M[0][2]*z;
+			double YM=M[1][0]*x + M[1][1]*y + M[1][2]*z;
+			double ZM=M[2][0]*x + M[2][1]*y + M[2][2]*z+sp->view.height;
 			double absx, absy;
 			if(sp->view.depth){
-				if(X <= 0) break;
-				absx=SCALE_X(sp->view.depth*Y/X);
-				absy=SCALE_Y(sp->view.depth*Z/X);
+				if(XM <= 0) break;
+				absx=SCALE_X(sp->view.depth*YM/XM);
+				absy=SCALE_Y(sp->view.depth*ZM/XM);
 				if(absx < -sp->width || absx > 2*sp->width ||
 				   absy < -sp->height || absy > 2*sp->height)
 					break;
 			}else{
-				absx=SCALE_X(X);
-				absy=SCALE_Y(Y);
+				absx=SCALE_X(XM);
+				absy=SCALE_Y(YM);
 			}
 			if(i){
 				sp->csegs[ix].x1 = (short) absx;
@@ -587,6 +598,7 @@ release_flow(ModeInfo * mi)
 		free(flows);
 		flows = (flowstruct *) NULL;
 	}
+	XSetClipMask(MI_DISPLAY(mi), MI_GC(mi), None);
 }
 
 void
