@@ -191,21 +191,20 @@ ModStruct   atlantis_description =
 
 static atlantisstruct *atlantis = (atlantisstruct *) NULL;
 
-static void
+#ifdef HAVE_XPM
+static Bool
 parse_image_data(ModeInfo *mi)
 {
-#ifdef HAVE_XPM
   atlantisstruct *ap = &atlantis[MI_SCREEN(mi)];
 
   if ((ap->texture = xpm_to_ximage (MI_DISPLAY(mi),
                                MI_VISUAL(mi),
                                MI_COLORMAP(mi),
                                sea_texture)) == None)
-    do_texture = False;
-#else
-  do_texture = False;
-#endif
+	return True;
+  return False;
 }
+#endif
 
 static void
 InitFishs(atlantisstruct * ap)
@@ -301,7 +300,17 @@ Init(ModeInfo * mi)
             glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient);
           }
 
-        if (ap->wire || !do_texture)
+#ifdef HAVE_XPM
+        if (ap->wire)
+		do_texture = False;
+        if (do_texture && !ap->texture) {
+        	do_texture = parse_image_data (mi);
+            	clear_gl_error();
+	}
+#else
+	do_texture = False;
+#endif
+        if (!do_texture)
           {
             glDisable(GL_TEXTURE_2D);
           }
@@ -311,36 +320,38 @@ Init(ModeInfo * mi)
             GLfloat t_plane[] = { 0, 0, 1, 0 };
             GLfloat scale = 0.0005;
 
-	    if (!ap->texture)
-              parse_image_data (mi);
-
-            clear_gl_error();
+#ifdef HAVE_XPM
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
                          ap->texture->width, ap->texture->height, 0,
                          GL_RGBA, GL_UNSIGNED_BYTE,
                          ap->texture->data);
-            check_gl_error("texture");
+            if (check_gl_error("texture")) {
+	       do_texture = False;
+               glDisable(GL_TEXTURE_2D);
+	    } else {
 
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 
-            glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+              glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
-            glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
-            glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
-            glTexGenfv(GL_S, GL_EYE_PLANE, s_plane);
-            glTexGenfv(GL_T, GL_EYE_PLANE, t_plane);
+              glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+              glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+              glTexGenfv(GL_S, GL_EYE_PLANE, s_plane);
+              glTexGenfv(GL_T, GL_EYE_PLANE, t_plane);
 
-            glEnable(GL_TEXTURE_GEN_S);
-            glEnable(GL_TEXTURE_GEN_T);
-            glEnable(GL_TEXTURE_2D);
+              glEnable(GL_TEXTURE_GEN_S);
+              glEnable(GL_TEXTURE_GEN_T);
+              glEnable(GL_TEXTURE_2D);
 
-            glMatrixMode(GL_TEXTURE);
-            glLoadIdentity();
-            glScalef(scale, scale, 1);
-            glMatrixMode(GL_MODELVIEW);
+              glMatrixMode(GL_TEXTURE);
+              glLoadIdentity();
+              glScalef(scale, scale, 1);
+              glMatrixMode(GL_MODELVIEW);
+	    }
+#endif
           }
 
 	InitFishs(ap);
@@ -393,7 +404,10 @@ clear_tank (atlantisstruct * ap)
           int size = 4 * (end - start);
           int i;
 
-          pixels = (unsigned char *) malloc (size);
+          if ((pixels = (unsigned char *) malloc (size)) == NULL) {
+		do_gradient = False;
+		return;
+	  }
           i = 0;
           while (i < size)
             {
@@ -413,11 +427,11 @@ clear_tank (atlantisstruct * ap)
           glTexImage1D(GL_TEXTURE_1D, 0, 4,
                        (end - start), 0,
                        GL_RGBA, GL_UNSIGNED_BYTE, pixels);
-          check_gl_error ("gradient texture");
+          if (check_gl_error ("gradient texture"))
+               return;
           glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
           glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 
-
           gradient_list = glGenLists(1);
           glNewList(gradient_list, GL_COMPILE);
 
@@ -457,7 +471,10 @@ clear_tank (atlantisstruct * ap)
             glEnable(GL_TEXTURE_2D);
 
           glEndList();
-          check_gl_error ("gradient list");
+          if (check_gl_error ("gradient list")) {
+      		do_gradient = False;
+	  	return;
+	  }
         }
 
       glCallList(gradient_list);
