@@ -294,7 +294,7 @@ sphere (GLfloat x, GLfloat y, GLfloat z, GLfloat diameter, Bool wire)
 }
 
 
-static void
+static Bool
 load_font (ModeInfo *mi, char *res, XFontStruct **fontP, GLuint *dlistP)
 {
 #ifdef STANDALONE
@@ -317,30 +317,40 @@ load_font (ModeInfo *mi, char *res, XFontStruct **fontP, GLuint *dlistP)
   
   clear_gl_error ();
   *dlistP = glGenLists ((GLuint) last+1);
-  check_gl_error ("glGenLists");
+  if (check_gl_error ("glGenLists"))
+	return False;
   /*-
    * PURIFY reports a cumulative memory leak on the next line with Mesa 3.4.1
    * on Solaris 2.X and SunOS 4.1.X. This can be fixed with a patch to Mesa
    * 3.4.1. OpenGL on Solaris does not have the memory leak.
    */
   glXUseXFont(id, first, last-first+1, *dlistP + first);
-  check_gl_error ("glXUseXFont");
+  if (check_gl_error ("glXUseXFont"))
+	return False;
 
   *fontP = f;
+  return True;
 }
 
 
-static void
+static Bool
 load_fonts (ModeInfo *mi)
 {
   molecule_configuration *mc = &mcs[MI_SCREEN(mi)];
 #ifdef STANDALONE
-  load_font (mi, (char *) "atomFont",  &mc->xfont1, &mc->font1_dlist);
-  load_font (mi, (char *) "titleFont", &mc->xfont2, &mc->font2_dlist);
+  if (!load_font (mi, (char *) "atomFont",  &mc->xfont1, &mc->font1_dlist)
+	return False;
+  if (!load_font (mi, (char *) "titleFont", &mc->xfont2, &mc->font2_dlist)
+	return False;
 #else
-  load_font (mi, (char *) "-*-times-bold-r-normal-*-240-*",  &mc->xfont1, &mc->font1_dlist);
-  load_font (mi, (char *) "-*-times-bold-r-normal-*-180-*", &mc->xfont2, &mc->font2_dlist);
+  if (!load_font (mi, (char *) "-*-times-bold-r-normal-*-240-*",
+		&mc->xfont1, &mc->font1_dlist))
+	return False;
+  if (!load_font (mi, (char *) "-*-times-bold-r-normal-*-180-*",
+		&mc->xfont2, &mc->font2_dlist))
+	return False;
 #endif
+  return True;
 }
 
 static void
@@ -1369,7 +1379,10 @@ init_molecule (ModeInfo *mi)
     reshape_molecule (mi, MI_WIDTH(mi), MI_HEIGHT(mi));
   }
 
-  load_fonts (mi);
+  if (!load_fonts (mi)) {
+	release_molecule(mi);
+	return;
+  }
   if (firstcall)
   startup_blurb (mi);
   cur_wire = MI_IS_WIREFRAME(mi);
@@ -1433,10 +1446,13 @@ draw_molecule (ModeInfo *mi)
 {
 /*  static time_t last = 0; */
   time_t now = time ((time_t *) 0);
-  molecule_configuration *mc = &mcs[MI_SCREEN(mi)];
+  molecule_configuration *mc;
   Display *dpy = MI_DISPLAY(mi);
   Window window = MI_WINDOW(mi);
 
+  if (mcs == NULL)
+	return;
+  mc = &mcs[MI_SCREEN(mi)];
   if (!mc->glx_context)
     return;
 
