@@ -2,7 +2,7 @@
 /* life3d --- Extension to Conway's game of Life, Carter Bays' S45/B5 3d life */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)life3d.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)life3d.c	5.07 2003/01/08 xlockmore";
 
 #endif
 
@@ -22,6 +22,9 @@ static const char sccsid[] = "@(#)life3d.c	5.00 2000/11/01 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 25-Jan-2003: Spawned a life3d.h
+ * 19-Jan-2003: added more gliders from http://www.cse.sc.edu/~bays
+ * 08-Jan-2003: Double buffering
  * 01-Nov-2000: Allocation checks
  * 10-May-1997: Compatible with xscreensaver
  * 18-Apr-1997: Memory leak fixed by Tom Schmidt <tschmidt@micron.com>
@@ -30,8 +33,6 @@ static const char sccsid[] = "@(#)life3d.c	5.00 2000/11/01 xlockmore";
  * 07-Dec-1994: used life.c and a DOS version of 3dlife
  * Copyright 1993 Anthony Wesley awesley@canb.auug.org.au found at
  * life.anu.edu.au /pub/complex_systems/alife/3DLIFE.ZIP
- * There is some flashing that was not in the original.  This is because
- * the direct video memory access garbage collection was not portable.
  *
  *
  * References:
@@ -64,13 +65,18 @@ static const char sccsid[] = "@(#)life3d.c	5.00 2000/11/01 xlockmore";
 
 #ifdef MODE_life3d
 
+#include "life3d.h"
+
 #if 1
 #define DEF_RULE  "G"		/* All rules with gliders */
 #else
 #define DEF_RULE  "P"		/* All rules with known patterns */
 #define DEF_RULE  "S45/B5"	/* "B5/S45" */
 #define DEF_RULE  "S567/B6"	/* "B6/S567" */
-/* There are no known gliders for S56/B5 or S67/B67 */
+#define DEF_RULE  "S56/B5"	/* "B5/S56" */
+#define DEF_RULE  "S678/B5"	/* "B5/S678" */
+#define DEF_RULE  "S67/B67"	/* "B67/S67" */
+/* There are no known gliders for S67/B67 */
 #endif
 
 static char *rule;
@@ -115,7 +121,6 @@ ModStruct   life3d_description =
 
 #define RT_ANGLE 90
 #define HALFRT_ANGLE 45
-#define MAXNEIGHBORS 27
 
 /* Store state of cell in top bit. Reserve low bits for count of living nbrs */
 #define Set3D(x,y,z) SetMem(lp,(unsigned int)x,(unsigned int)y,(unsigned int)z,ON)
@@ -144,8 +149,6 @@ if (!AddToList(lp,(unsigned int)x,(unsigned int)y,(unsigned int)z)) return False
 #define GREEN 2
 #define BLUE 3
 #define WHITE 4
-#define NUMPTS 42
-#define NUMFILEPTS (2*NUMPTS)
 
 typedef struct _CellList {
 	unsigned char x, y, z;	/* Location in world coordinates */
@@ -156,10 +159,6 @@ typedef struct _CellList {
 	struct _CellList *priority;
 } CellList;
 
-typedef struct {
-	int         survival, birth;
-} paramstruct;
-
 typedef struct {
 	Bool        painted;
 	paramstruct param;
@@ -170,6 +169,7 @@ typedef struct {
 	int         nstacks, nrows, ncolumns;
 	int         memstart;
 	char        visible;
+	int         noChangeCount;
 	unsigned char *base[BASESIZE];
 	double      A, B, C, F;
 	int         width, height;
@@ -179,6 +179,7 @@ typedef struct {
 	CellList   *ptrhead, *ptrend, eraserhead, eraserend;
 	CellList   *buckethead[NBUCKETS], *bucketend[NBUCKETS];		/* comfortable upper b */
 	Bool        wireframe;
+	Pixmap      dbuf;
 } life3dstruct;
 
 static life3dstruct *life3ds = (life3dstruct *) NULL;
@@ -187,1032 +188,6 @@ static paramstruct input_param;
 static Bool allPatterns = False, allGliders = False;
 static char *filePattern = (char *) NULL;
 
-/*-
- * S45/B5 life is probably the best
- * S567/B6 life has gliders like Conway's 2d S23/B3 life
- * There are no known gliders for S56/B5 or S67/B67,
- * so the others may be better
- */
-
-static char patterns_S45B5[][3 * NUMPTS + 1] =
-{
-#if 0
-/* Still life */
-	{			/* V */
-		-1, -1, -1, 0, -1, -1,
-		-1, 0, -1, 0, 0, -1,
-
-		-1, -1, 0, 0, -1, 0,
-		127
-	},
-	{			/* CROSS */
-		0, 0, -1,
-
-		0, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-
-		0, 0, 1,
-		127
-	},
-	{			/* PILLAR */
-		0, -1, -1,
-		-1, 0, -1, 1, 0, -1,
-		0, 1, -1,
-
-		0, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-		127
-	},
-#endif
-	{			/* BLINKER P2 */
-		0, 0, -1,
-
-		0, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-
-		-1, 0, 1, 1, 0, 1,
-		127
-	},
-	{			/* DOUBLE BLINKER P2 */
-		0, -1, -1,
-		0, 1, -1,
-
-		0, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-
-		-1, 0, 1, 1, 0, 1,
-		127
-	},
-	{			/* TRIPLE BLINKER 1 P2 */
-		-1, -1, -2,
-		-1, 0, -2,
-
-		-2, -1, -1,
-		1, 0, -1,
-		-1, 1, -1, 0, 1, -1,
-
-		-1, -2, 0, 0, -2, 0,
-		-2, -1, 0, 1, 0, 0,
-
-		0, -1, 1,
-		0, 0, 1,
-		127
-	},
-	{			/* TRIPLE BLINKER 2 P2 */
-		-1, -1, -2, 0, -1, -2,
-
-		0, -2, -1,
-		1, -1, -1,
-		1, 0, -1,
-		-1, 1, -1,
-
-		0, -2, 0,
-		-2, -1, 0,
-		-2, 0, 0,
-		-1, 1, 0,
-
-		-1, 0, 1, 0, 0, 1,
-		127
-	},
-	{			/* THREE HALFS BLINKER  P2 */
-		0, -1, -1,
-		-1, 0, -1, 1, 0, -1,
-
-		-1, -1, 0,
-		1, 0, 0,
-		-1, 1, 0, 0, 1, 0,
-
-		0, -1, 1,
-		0, 0, 1,
-		127
-	},
-	{			/* PUFFER P4 */
-		0, -1, -1,
-		0, 0, -1,
-
-		0, -2, 0,
-		-1, -1, 0, 1, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-		127
-	},
-	{			/* PINWHEEL P4 */
-		-1, 1, -1, 0, 1, -1,
-
-		-1, -1, 0, 0, -1, 0,
-		-2, 0, 0, 1, 0, 0,
-
-		-1, 0, 1, 0, 0, 1,
-		-1, 1, 1, 0, 1, 1,
-		127
-	},
-	{			/* HEART P4 */
-		-1, -1, -1,
-		-1, 0, -1, 0, 0, -1,
-
-		0, -1, 0,
-		-2, 0, 0, 1, 0, 0,
-
-		-1, -1, 1,
-		-1, 1, 1, 0, 1, 1,
-		127
-	},
-	{			/* ARROW P4 */
-		0, -1, -1,
-		0, 0, -1,
-
-		0, -2, 0,
-		1, -1, 0,
-		1, 0, 0,
-		0, 1, 0,
-
-		-1, -1, 1,
-		-1, 0, 1,
-		127
-	},
-	{			/* ROTOR P2 */
-		0, -1, -1,
-		0, 0, -1,
-
-		0, -2, 0,
-		-1, -1, 0, 1, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-
-		0, -1, 1,
-		0, 0, 1,
-		127
-	},
-	{			/* BRONCO P4 */
-		0, -1, -1,
-		0, 0, -1,
-
-		0, -2, 0,
-		-1, -1, 0, 1, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-
-		-1, -1, 1,
-		-1, 0, 1,
-		127
-	},
-	{			/* TRIPUMP P2 */
-		0, -2, -2,
-		-2, -1, -2, -1, -1, -2, 0, -1, -2,
-
-		0, -2, -1,
-		-2, 0, -1,
-		-2, 1, -1, -1, 1, -1,
-
-		1, -2, 0,
-		1, -1, 0,
-		1, 0, 0,
-		-1, 1, 0,
-
-		0, 0, 1, 1, 0, 1,
-		-1, 1, 1,
-		127
-	},
-	{			/* WINDSHIELDWIPER (HELICOPTER) P2 */
-		-2, -1, -2, -1, -1, -2,
-		0, 0, -2,
-
-		-1, -2, -1,
-		-2, -1, -1,
-		1, 0, -1,
-		-1, 1, -1, 0, 1, -1, 1, 1, -1,
-
-		0, -2, 0,
-		-2, -1, 0, 1, -1, 0,
-
-		0, -2, 1,
-		0, -1, 1,
-		0, 0, 1,
-		127
-	},
-	{			/* WALTZER P6 */
-		-2, -1, -1, -1, -1, -1, 0, -1, -1,
-		-2, 0, -1,
-		-1, 1, -1, 0, 1, -1,
-
-		-2, -1, 0,
-		-2, 0, 0, 1, 0, 0,
-		1, 1, 0,
-
-		-1, 0, 1, 0, 0, 1,
-		0, 1, 1,
-		127
-	},
-	{			/* BIG WALTZER P6 */
-		0, -1, -1, 1, -1, -1,
-		-1, 0, -1, 0, 0, -1,
-		-1, 1, -1,
-
-		0, -2, 0, 1, -2, 0,
-		-2, 0, 0,
-		-2, 1, 0,
-
-		0, -1, 1, 1, -1, 1,
-		-1, 0, 1, 0, 0, 1,
-		-1, 1, 1,
-		127
-	},
-	{			/* SEESAW P2 */
-		0, 0, -2,
-
-		-2, -1, -1, -1, -1, -1,
-		-2, 0, -1,
-		0, 1, -1,
-
-		-1, -1, 0,
-		1, 0, 0,
-		0, 1, 0, 1, 1, 0,
-
-		-1, 0, 1,
-		127
-	},
-	{			/* COLLISION -> REDIRECTION */
-		-3, 5, -7, 0, 5, -7,
-		-3, 4, -7, 0, 4, -7,
-		-2, 3, -7, -1, 3, -7,
-
-		-2, 5, -6, -1, 5, -6,
-		-2, 4, -6, -1, 4, -6,
-
-		1, -2, 1,
-
-		1, -3, 2,
-		0, -2, 2, 2, -2, 2,
-		1, -1, 2,
-
-		0, -2, 3, 2, -2, 3,
-		127
-	},
-	{			/* COLLISION -> SEESAW P2 */
-		-4, -6, -7, -1, -6, -7,
-		-4, -5, -7, -1, -5, -7,
-		-3, -4, -7, -2, -4, -7,
-
-		-3, -6, -6, -2, -6, -6,
-		-3, -5, -6, -2, -5, -6,
-
-		1, 4, 6, 2, 4, 6,
-		0, 5, 6, 3, 5, 6,
-		0, 6, 6, 3, 6, 6,
-
-		1, 5, 5, 2, 5, 5,
-		1, 6, 5, 2, 6, 5,
-		127
-	}
-};
-
-static char patterns_S567B6[][3 * NUMPTS + 1] =
-{
-	{			/* KNIFE-SWITCH BLINKER */
-		0, -1, -1,
-		-1, 0, -1, 0, 0, -1, 1, 0, -1,
-
-		0, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 0, 1,
-		127,
-	},
-	{			/* CLOCK */
-		0, -1, -2,
-		0, 0, -2,
-
-		-2, -1, -1, 0, -1, -1,
-		-2, 0, -1, 0, 0, -1,
-
-		-1, -1, 0, 1, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-
-		-1, -1, 1,
-		-1, 0, 1,
-		127,
-	},
-	{			/* HALF BLINKERS */
-		0, -1, -2,
-		0, 0, -2,
-
-		-1, -1, -1, 1, -1, -1,
-		-1, 0, -1, 1, 0, -1,
-		0, 1, -1,
-
-		-1, -2, 0, 0, -2, 0, 1, -2, 0,
-		-2, -1, 0, 2, -1, 0,
-		-2, 0, 0, 2, 0, 0,
-		0, 1, 0,
-
-		-1, -1, 1, 1, -1, 1,
-		-1, 0, 1, 1, 0, 1,
-		0, 1, 1,
-
-		0, -1, 2,
-		0, 0, 2,
-		127,
-	},
-	{			/* MUTANT HALF BLINKERS */
-		0, -1, -2,
-		0, 0, -2,
-
-		0, -2, -1,
-		-1, -1, -1, 1, -1, -1,
-		-1, 0, -1, 1, 0, -1,
-		0, 1, -1,
-
-		-1, -2, 0, 1, -2, 0,
-		-2, -1, 0, 2, -1, 0,
-		-2, 0, 0, 2, 0, 0,
-		0, 1, 0,
-
-		0, -2, 1, 1, -2, 1,
-		-1, -1, 1, 2, -1, 1,
-		-1, 0, 1, 2, 0, 1,
-		0, 1, 1,
-
-		0, -1, 2, 1, -1, 2,
-		0, 0, 2,
-		127,
-	},
-	{			/* FUSE */
-		0, 0, 4,
-
-		0, 0, 3,
-
-		0, -2, 2,
-		-1, -1, 2, 1, -1, 2,
-		-2, 0, 2, 2, 0, 2,
-		-1, 1, 2, 1, 1, 2,
-		0, 2, 2,
-
-		0, -2, 1,
-		-1, -1, 1, 1, -1, 1,
-		-2, 0, 1, 2, 0, 1,
-		-1, 1, 1, 1, 1, 1,
-		0, 2, 1,
-
-		0, 0, 0,
-
-		0, 0, -1,
-
-		0, -2, -2,
-		-1, -1, -2, 1, -1, -2,
-		-2, 0, -2, 2, 0, -2,
-		-1, 1, -2, 1, 1, -2,
-		0, 2, -2,
-
-		0, -2, -3,
-		-1, -1, -3, 1, -1, -3,
-		-2, 0, -3, 2, 0, -3,
-		-1, 1, -3, 1, 1, -3,
-		0, 2, -3,
-
-		-1, 0, -4, 0, 0, -4, 1, 0, -4,
-		127,
-	},
-	{			/* 2 PTS OF STAR OF DAVID */
-		1, 0, -3, 2, 0, -3,
-
-		1, -1, -2, 2, -1, -2,
-		0, 0, -2,
-		1, 1, -2, 2, 1, -2,
-
-		1, 0, -1, 2, 0, -1,
-
-		-2, 0, 0, 0, 0, 0,
-
-		-2, -1, 1,
-		-3, 0, 1, -1, 0, 1,
-		-2, 1, 1,
-
-		-2, -1, 2,
-		-3, 0, 2, -1, 0, 2,
-		-2, 1, 2,
-		127,
-	},
-	{			/* TRIPLE BLINKER 1 */
-		0, -1, -2,
-		0, 0, -2,
-
-		-1, -1, -1, 1, -1, -1,
-		-2, 0, -1,
-		-1, 1, -1, 0, 1, -1,
-
-		-1, -2, 0, 0, -2, 0,
-		1, -1, 0,
-		-2, 0, 0, 0, 0, 0,
-
-		-1, -1, 1,
-		-1, 0, 1,
-		127,
-	},
-	{			/* TRIPLE BLINKER 2 */
-		-1, -1, -2, 0, -1, -2,
-
-		-1, -2, -1,
-		-2, -1, -1, -1, -1, -1,
-		-2, 0, -1,
-		0, 1, -1,
-
-		-1, -2, 0,
-		1, -1, 0,
-		0, 0, 0, 1, 0, 0,
-		0, 1, 0,
-
-		-1, 0, 1, 0, 0, 1,
-		127,
-	},
-	{			/* TRIKNOT */
-		0, 0, -1, 1, 0, -1,
-
-		-1, -1, 0, 0, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-
-		0, 0, 1,
-		0, 1, 1,
-		127,
-	},
-	{			/* SEARCHLIGHT */
-		-1, -1, -2, 0, -1, -2,
-		-1, 0, -2, 0, 0, -2,
-
-		-1, -2, -1, 0, -2, -1,
-		-2, -1, -1, 1, -1, -1,
-		-2, 0, -1, 1, 0, -1,
-		0, 1, -1,
-
-		-1, -2, 0, 0, -2, 0,
-		1, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-
-		-1, -1, 1, 0, -1, 1,
-		127,
-	},
-	{			/* POLE DRIVER */
-		-1, -1, -1, 0, -1, -1,
-		-1, 0, -1, 1, 0, -1,
-		0, 1, -1, 1, 1, -1,
-
-		0, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0,
-
-		0, 0, 1,
-		127,
-	},
-	{			/* LITTLE STAR */
-		0, -1, -1,
-		-1, 0, -1, 1, 0, -1,
-		-1, 1, -1, 0, 1, -1, 1, 1, -1,
-
-		-1, -1, 0, 1, -1, 0,
-		-1, 1, 0, 1, 1, 0,
-
-		-1, -1, 1, 0, -1, 1, 1, -1, 1,
-		-1, 0, 1, 1, 0, 1,
-		0, 1, 1,
-		127,
-	},
-	{			/* JAWS */
-		0, -1, -2,
-		-1, 0, -2, 1, 0, -2,
-		0, 1, -2,
-
-		0, -2, -1,
-		-1, -1, -1, 1, -1, -1,
-		-1, 0, -1, 2, 0, -1,
-		0, 1, -1, 1, 1, -1,
-
-		-1, -2, 0, 1, -2, 0,
-		-1, -1, 0, 2, -1, 0,
-		1, 1, 0, 2, 1, 0,
-
-		0, -2, 1,
-		0, -1, 1, 1, -1, 1,
-		1, 0, 1, 2, 0, 1,
-
-		-2, 1, 1,
-		-2, 2, 1, -1, 2, 1,
-
-		-2, 1, 2, -1, 1, 2,
-		-2, 2, 2, -1, 2, 2,
-		127,
-	},
-	{			/* NEAR SHIP */
-		-2, -1, -2, -1, -1, -2,
-		-2, 0, -2, -1, 0, -2,
-
-		-2, -1, -1, -1, -1, -1,
-		-2, 0, -1, -1, 0, -1,
-
-		0, -1, 0,
-		0, 0, 0,
-
-		-2, -1, 1, -1, -1, 1,
-		-2, 0, 1, -1, 0, 1,
-
-		-2, -1, 2, -1, -1, 2,
-		-2, 0, 2, -1, 0, 2,
-		127
-	}
-};
-
-/*-
- * Many names of S56/B5 & S67/B67 are made up by David Bagley.
- * They are listed in * order given by Carter Bays all p2 except "H"
- */
-static char patterns_S56B5[][3 * NUMPTS + 1] =
-{
-	{			/* SEESAW */
-		0, -1, -1,
-		-1, 0, -1, 0, 0, -1,
-
-		0, 0, 0, 1, 0, 0,
-		0, 1, 0,
-		127
-	},
-	{			/* PROP */
-		-1, 0, -1, 0, 0, -1, 1, 0, -1,
-
-		0, -1, 0,
-		0, 0, 0,
-		0, 1, 0,
-		127
-	},
-	{			/*  */
-		-1, 0, -1, 0, 0, -1,
-
-		0, -1, 0,
-		0, 0, 0, 1, 0, 0,
-		0, 1, 0,
-		127
-	},
-	{			/* COLUMN */
-		0, 0, -6,
-
-		0, 0, -5,
-
-		0, -1, -4,
-		-1, 0, -4, 0, 0, -4, 1, 0, -4,
-		0, 1, -4,
-
-		0, -1, -1,
-		-1, 0, -1, 0, 0, -1, 1, 0, -1,
-		0, 1, -1,
-
-		0, 0, 0,
-
-		0, 0, 1,
-
-		0, 0, 2,
-
-		0, 0, 3,
-
-		0, -1, 4,
-		-1, 0, 4, 0, 0, 4, 1, 0, 4,
-		0, 1, 4,
-		127
-	},
-	{			/* FLIPPING C */
-		-1, 0, -1, 1, 0, -1,
-
-		-1, 0, 0, 0, 0, 0, 1, 0, 0,
-		127
-	},
-	{			/* SLIDING BLOCKS  */
-		-1, 0, -1, 0, 0, -1,
-		-1, 1, -1, 0, 1, -1,
-
-		-1, -1, 0, 0, -1, 0,
-		0, 0, 0,
-		127
-	},
-	{			/*  */
-		-1, -1, -1,
-		-1, 1, -1,
-
-		-1, 0, 0, 0, 0, 0, 1, 0, 0,
-
-		-1, -1, 1,
-		-1, 1, 1,
-		127
-	},
-	{			/* Y */
-		1, -1, -1,
-		-1, 0, -1,
-		0, 1, -1,
-
-		1, -1, 0,
-		-1, 0, 0, 0, 0, 0,
-		0, 1, 0,
-		127
-	},
-	{			/* PUMP */
-		0, -1, -1,
-		-1, 0, -1, 1, 0, -1,
-		0, 1, -1,
-
-		-1, -1, 0, 0, -1, 0,
-		-1, 0, 0, 1, 0, 0,
-		0, 1, 0, 1, 1, 0,
-		127
-	},
-	{			/* WALKER */
-		-1, -1, -1, 0, -1, -1,
-		1, 0, -1,
-
-		0, -1, 0, 1, -1, 0,
-		-1, 0, 0, 0, 0, 0,
-		127
-	},
-	{			/* SWITCH */
-		2, -1, -2,
-		1, 0, -2, 2, 0, -2,
-
-		1, -1, -1, 2, -1, -1,
-		1, 0, -1,
-
-		1, 0, 0,
-
-		-1, -1, 1,
-		-2, 0, 1, -1, 0, 1, 0, 0, 1,
-
-		-2, -1, 2, -1, -1, 2,
-		-2, 0, 2,
-		127
-	},
-	{			/* HOPPER */
-		-1, -1, -1, 0, -1, -1, 1, -1, -1,
-
-		0, -1, 0,
-		-1, 0, 0, 0, 0, 0, 1, 0, 0,
-		127
-	},
-	{			/* PUSHUPS */
-		-1, -2, -2,
-
-		-1, -2, -1, 0, -2, -1,
-		-1, -1, -1, 0, -1, -1,
-
-		-1, 0, 0, 0, 0, 0,
-
-		-1, 0, 1, 0, 0, 1,
-		-1, 1, 1,
-		127
-	},
-	{			/* PUSHER */
-		0, -2, -2,
-		0, -1, -2,
-
-		-1, -2, -1,
-		-1, -1, -1, 0, -1, -1,
-
-		-1, 0, 0, 0, 0, 0,
-		0, 1, 0,
-
-		-1, 0, 1,
-		0, 1, 1,
-		127
-	},
-	{			/* CAPACITOR */
-		0, -1, -2,
-		-1, 0, -2, 0, 0, -2,
-
-		0, 0, -1, 1, 0, -1,
-		0, 1, -1, 1, 1, -1,
-
-		0, 0, 1, 1, 0, 1,
-		0, 1, 1, 1, 1, 1,
-
-		0, -1, 2,
-		-1, 0, 2, 0, 0, 2,
-		127
-	},
-	{			/* CORNER 1 */
-		-1, -3, -3,
-		-1, -2, -3, 0, -2, -3,
-		1, -1, -3,
-		1, 0, -3, 2, 0, -3,
-
-		-1, -3, -2,
-		0, -2, -2,
-		1, -1, -2,
-		2, 0, -2,
-
-		-3, -3, -1, -2, -3, -1,
-		-3, -2, -1,
-		2, 1, -1,
-		1, 2, -1, 2, 2, -1,
-
-		-3, -2, 0, -2, -2, 0,
-		1, 1, 0,
-		1, 2, 0,
-
-		-3, -1, 1, -2, -1, 1,
-		-3, 0, 1,
-		0, 1, 1,
-		-1, 2, 1, 0, 2, 1,
-
-		-3, 0, 2, -2, 0, 2,
-		-1, 1, 2,
-		-1, 2, 2,
-		127
-	},
-	{			/* CORNER 2 */
-		-2, -2, -3, -1, -2, -3,
-		-2, -1, -3,
-		2, 1, -3,
-		1, 2, -3, 2, 2, -3,
-
-		-3, -2, -2, -1, -2, -2,
-		-3, -1, -2, 0, -1, -2,
-		1, 0, -2,
-		2, 1, -2,
-		1, 3, -2, 2, 3, -2,
-
-		-3, -2, -1, -2, -2, -1,
-		0, -1, -1,
-		1, 0, -1,
-		2, 2, -1,
-		2, 3, -1,
-
-		-2, -1, 0, -1, -1, 0,
-		1, 1, 0,
-		1, 2, 0,
-
-		-2, 0, 1, -1, 0, 1,
-		0, 1, 1,
-		-3, 2, 1, 0, 2, 1,
-		-2, 3, 1,
-
-		-3, 1, 2, -2, 1, 2,
-		-3, 2, 2, -1, 2, 2,
-		-2, 3, 2, -1, 3, 2,
-		127
-	},
-	{			/* RUNNER */
-		-1, -2, -1, 0, -2, -1,
-		0, -1, -1,
-
-		-1, -1, 0, 0, -1, 0,
-
-		-1, 0, 1, 0, 0, 1,
-		0, 1, 1,
-
-		-1, 1, 2, 0, 1, 2,
-		127
-	},
-	{			/* BACKWARDS RUNNER */
-		-1, -2, -1, 0, -2, -1,
-		0, -1, -1,
-
-		-1, -1, 0, 0, -1, 0,
-
-		-1, 0, 1, 0, 0, 1,
-		-1, 1, 1,
-
-		-1, 0, 2, 0, 0, 2,
-		127
-	},
-	{			/* FLIPPING H */
-		-1, 0, -1, 1, 0, -1,
-
-		-1, 0, 0, 0, 0, 0, 1, 0, 0,
-
-		-1, 0, 1, 1, 0, 1,
-		127
-	},
-	{			/* EAGLE */
-		0, -2, -1, 1, -2, -1,
-		0, -1, -1, 1, -1, -1,
-		0, 1, -1, 1, 1, -1,
-		0, 2, -1, 1, 2, -1,
-
-		0, -1, 0,
-		-1, 0, 0,
-		0, 1, 0,
-		127
-	},
-	{			/* VARIANT 1 */
-		1, -1, -2,
-		0, 0, -2,
-
-		0, -1, -1,
-		0, 0, -1, 1, 0, -1,
-
-		-1, -1, 0,
-
-		0, -1, 1,
-		0, 0, 1, 1, 0, 1,
-
-		1, -1, 2,
-		0, 0, 2,
-		127
-	},
-	{			/* VARIANT 2 */
-		0, -1, -2,
-		1, 0, -2,
-
-		0, -1, -1, 1, -1, -1,
-		0, 0, -1,
-
-		-1, 0, 0,
-
-		0, -1, 1,
-		0, 0, 1, 1, 0, 1,
-
-		1, -1, 2,
-		0, 0, 2,
-		127
-	},
-	{			/* VARIANT 3 */
-		1, -1, -2,
-		0, 0, -2,
-
-		0, -1, -1, 1, -1, -1,
-		0, 0, -1,
-
-		-1, -1, 0,
-
-		0, -1, 1,
-		0, 0, 1, 1, 0, 1,
-
-		1, -1, 2,
-		0, 0, 2,
-		127
-	},
-	{			/* VARIANT 4 */
-		1, -1, -2,
-		0, 0, -2,
-
-		0, -1, -1, 1, -1, -1,
-		0, 0, -1,
-
-		-1, 0, 0,
-
-		0, -1, 1,
-		0, 0, 1, 1, 0, 1,
-
-		1, -1, 2,
-		0, 0, 2,
-		127
-	},
-	{			/* SQUID */
-		-2, -1, -2, -1, -1, -2,
-		1, 1, -2,
-		1, 2, -2,
-
-		-2, -1, -1, -1, -1, -1, 0, -1, -1,
-		1, 0, -1,
-		1, 1, -1,
-		1, 2, -1,
-
-		-1, -1, 0,
-		1, 1, 0,
-
-		-1, 0, 1,
-		-2, 1, 1, -1, 1, 1, 0, 1, 1,
-		-2, 2, 1, -1, 2, 1,
-		127
-	},
-	{			/* ROWER */
-		0, 1, -2,
-
-		-1, -1, -1, 1, -1, -1,
-		-2, 0, -1, -1, 0, -1, 1, 0, -1, 2, 0, -1,
-
-		-1, 0, 0, 1, 0, 0,
-		-1, 1, 0, 1, 1, 0,
-
-		0, -1, 1,
-		127
-	},
-	{			/* FLIP */
-		-1, -2, -2,
-		0, -1, -2,
-		0, 0, -2,
-
-		-1, -2, -1,
-		-2, -1, -1, 1, -1, -1,
-		1, 0, -1,
-		1, 1, -1,
-
-		-1, -2, 0,
-		-2, -1, 0,
-		-1, 1, 0, 0, 1, 0,
-
-		-2, 0, 1, -1, 0, 1, 0, 0, 1,
-		127
-	}
-};
-static char patterns_S67B67[][3 * NUMPTS + 1] =
-{
-	{			/* WALKING BOX */
-		-1, -1, -2,
-		0, 0, -2,
-
-		-1, -2, -1, 0, -2, -1,
-		-2, -1, -1, 1, -1, -1,
-		-2, 0, -1, 1, 0, -1,
-		-1, 1, -1, 0, 1, -1,
-
-		-1, -2, 0, 0, -2, 0,
-		-2, -1, 0, 1, -1, 0,
-		-2, 0, 0, 1, 0, 0,
-		-1, 1, 0, 0, 1, 0,
-
-		-1, -1, 1,
-		0, 0, 1,
-		127
-	},
-	{			/* WALKER */
-		0, -1, -1,
-		-1, 0, -1,
-
-		-1, -1, 0, 0, -1, 0,
-		-1, 0, 0, 0, 0, 0,
-
-		0, -1, 1,
-		-1, 0, 1,
-		127
-	},
-	{			/* S */
-		-1, -1, -1, 0, -1, -1,
-		-1, 0, -1, 0, 0, -1,
-
-		-1, 0, 0, 0, 0, 0,
-		-1, 1, 0, 0, 1, 0,
-		127
-	},
-	{			/* BACKWARDS ARROW */
-		-1, 0, -1, 0, 0, -1,
-
-		-1, 0, 0, 0, 0, 0,
-		-1, 1, 0, 0, 1, 0,
-
-		-1, -1, 1, 0, -1, 1,
-		127
-	},
-	{			/* SPINING BOX */
-		-1, -1, -2, 0, -1, -2,
-		-1, 0, -2, 0, 0, -2,
-
-		0, -2, -1,
-		-2, -1, -1, 1, -1, -1,
-		-2, 0, -1,
-		-1, 1, -1, 0, 1, -1,
-
-		-1, -2, 0, 0, -2, 0,
-		-2, -1, 0, 1, -1, 0,
-		-2, 0, 0, 1, 0, 0,
-		-1, 1, 0, 0, 1, 0,
-
-		-1, -1, 1, 0, -1, 1,
-		0, 0, 1,
-		127
-	},
-	{			/* FLIPPING T */
-		0, 0, -1,
-
-		0, 0, 0,
-
-		0, -1, 1,
-		-1, 0, 1, 0, 0, 1, 1, 0, 1,
-		-1, 1, 1, 0, 1, 1,
-		127
-	}
-};
-
-static int  patterns_rules[] =
-{
-	(sizeof patterns_S45B5 / sizeof patterns_S45B5[0]),
-	(sizeof patterns_S567B6 / sizeof patterns_S567B6[0]),
-	(sizeof patterns_S56B5 / sizeof patterns_S56B5[0]),
-	(sizeof patterns_S67B67 / sizeof patterns_S67B67[0])
-};
-
-static paramstruct param_rules[] =
-{
-	{0x30, 0x20},
-	{0xE0, 0x40},
-	{0x60, 0x20},
-	{0xC0, 0xC0}
-};
-
-#define LIFE_S45B5 0
-#define LIFE_S567B6 1
-#define LIFE_GLIDERS 2		/* GLIDER rules are first in param_rules */
-#define LIFE_S56B5 2
-#define LIFE_S67B67 3
-#define LIFE_RULES (sizeof patterns_rules / sizeof patterns_rules[0])
-
 static int
 codeToPatternedRule(paramstruct param)
 {
@@ -1586,6 +561,7 @@ RunLife3D(life3dstruct * lp)
 	unsigned int x, y, z, xc, yc, zc;
 	int         c;
 	CellList   *ptr, *ptrnextcell;
+	Bool visible = False;
 
 	/* Step 1 - Add 1 to all neighbours of living cells. */
 	ptr = lp->ptrhead;
@@ -1618,6 +594,7 @@ RunLife3D(life3dstruct * lp)
 						if (c) {
 							if (CellState3D(c) == OFF) {
 								if (lp->param.birth & (1 << CellNbrs3D(c))) {
+									visible = True;
 									SetList3D(x, y, z);
 								} else {
 									if (!Reset3D(x, y, z))
@@ -1639,6 +616,10 @@ RunLife3D(life3dstruct * lp)
 		ptr = ptrnextcell;
 	}
 	ClearMem(lp);
+	if (visible)
+		lp->noChangeCount = 0;
+	else
+		lp->noChangeCount++;
 	return True;
 }
 
@@ -1715,6 +696,9 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 			case LIFE_S56B5:
 				patptr = &patterns_S56B5[pattern][0];
 				break;
+			case LIFE_S678B5:
+				patptr = &patterns_S678B5[pattern][0];
+				break;
 			case LIFE_S67B67:
 				patptr = &patterns_S67B67[pattern][0];
 				break;
@@ -1857,18 +841,20 @@ DrawFace(ModeInfo * mi, int color, XPoint * cubepts,
 	facepts[3] = cubepts[p4];
 	facepts[4] = cubepts[p1];
 
-	if (!lp->wireframe) {
-		XSetForeground(display, gc, lp->color[color]);
-		XFillPolygon(display, MI_WINDOW(mi), gc, facepts, 4,
-			     Convex, CoordModeOrigin);
-	}
+	XSetForeground(display, gc, lp->color[color]);
 	if (lp->wireframe) {
-		XSetForeground(display, gc, lp->color[color]);
-	} else if (color == BLACK || MI_NPIXELS(mi) <= 2) {
+		/* no double buffering needed */
+		XDrawLines(display, MI_WINDOW(mi), gc,
+			facepts, 5, CoordModeOrigin);
+		return;
+	}
+	XFillPolygon(display, lp->dbuf, gc, facepts, 4,
+		     Convex, CoordModeOrigin);
+	if (color == BLACK || MI_NPIXELS(mi) <= 2) {
 		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	} else
 		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
-	XDrawLines(display, MI_WINDOW(mi), gc, facepts, 5, CoordModeOrigin);
+	XDrawLines(display, lp->dbuf, gc, facepts, 5, CoordModeOrigin);
 }
 
 #define LEN 0.45
@@ -1945,9 +931,17 @@ DrawScreen(ModeInfo * mi)
 	CellList   *ptr;
 	CellList   *eraserptr;
 	int         i;
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
 
 	SortList(lp);
 
+	if (!lp->wireframe) {
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+		XFillRectangle(display, lp->dbuf, gc, 0,0,
+			lp->width, lp->height);
+	}
+
 	/* Erase dead cubes */
 	eraserptr = lp->eraserhead.next;
 	while (eraserptr != &lp->eraserend) {
@@ -1967,6 +961,11 @@ DrawScreen(ModeInfo * mi)
 			/* ++count; */
 		}
 	}
+	XFlush(display);
+	if (!lp->wireframe) {
+		XCopyArea(display, lp->dbuf, MI_WINDOW(mi), gc,
+			0, 0, lp->width, lp->height, 0, 0);
+	}
 #if 0
 	{
 		int         count = 0, v = 0;
@@ -1994,6 +993,7 @@ shooter(life3dstruct * lp)
 	r2 = lp->nrows / 2;
 	s2 = lp->nstacks / 2;
 	r = NRAND(3);
+r = 1;
 	if (!r) {
 		hsp = NRAND(V2) + c2 - V2 / 2;
 		vsp = (LRAND() & 1) ? r2 - V : r2 + V;
@@ -2026,6 +1026,21 @@ shooter(life3dstruct * lp)
 			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 2 * aoff);
 			SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 2 * aoff);
 			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 2 * aoff);
+		} else if (lp->patterned_rule == LIFE_S56B5) {
+			SetList3D(hsp + 2 * hoff, vsp + 4 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 4 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 4 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 4 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 3 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 4 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 3 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 3 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 3 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 4 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 4 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 4 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 4 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 4 * aoff);
 		}
 	} else if (r == 1) {
 		hsp = (LRAND() & 1) ? c2 - V : c2 + V;
@@ -2059,6 +1074,21 @@ shooter(life3dstruct * lp)
 			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 2 * hoff, vsp + 2 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 2 * hoff, vsp + 2 * voff, asp + 1 * aoff);
+		} else if (lp->patterned_rule == LIFE_S56B5) {
+			SetList3D(hsp + 4 * hoff, vsp + 0 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 4 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 4 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 4 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 3 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 4 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 3 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 3 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 3 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 4 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 4 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 4 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 4 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 4 * voff, asp + 0 * aoff);
 		}
 	} else {
 		hsp = (LRAND() & 1) ? c2 - V : c2 + V;
@@ -2092,16 +1122,42 @@ shooter(life3dstruct * lp)
 			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 2 * aoff);
 			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+		} else if (lp->patterned_rule == LIFE_S56B5) {
+			SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 4 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 4 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 4 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 4 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 3 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 4 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 3 * aoff);
+			SetList3D(hsp + 3 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 3 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 4 * hoff, vsp + 2 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 4 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 4 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 4 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 4 * hoff, vsp + 0 * voff, asp + 1 * aoff);
 		}
 	}
 	return True;
 }
 
+void free_life3d(Display *display, life3dstruct *lp)
+{
+	if (lp->eraserhead.next != NULL)
+		End3D(lp);
+        if (lp->dbuf != None) {
+		XFreePixmap(display, lp->dbuf);
+		lp->dbuf = None;
+	}
+}
+
 void
 init_life3d(ModeInfo * mi)
 {
 	int         i, npats;
 	life3dstruct *lp;
+	Display *display = MI_DISPLAY(mi);
 
 	if (life3ds == NULL) {
 		if ((life3ds = (life3dstruct *) calloc(MI_NUM_SCREENS(mi),
@@ -2160,10 +1216,11 @@ init_life3d(ModeInfo * mi)
 	lp->width = MI_WIDTH(mi);
 	lp->height = MI_HEIGHT(mi);
 	lp->memstart = 1;
+	lp->noChangeCount = False;
 	/*lp->tablesMade = 0; */
 
 	if (MI_IS_FULLRANDOM(mi)) {
-		lp->wireframe = (Bool) (LRAND() & 1);
+		lp->wireframe = (NRAND(4) == 0);
 	} else {
 		lp->wireframe = MI_IS_WIREFRAME(mi);
 	}
@@ -2201,6 +1258,18 @@ init_life3d(ModeInfo * mi)
 		}
 	}
 
+	if (lp->dbuf != None) {
+		XFreePixmap(display, lp->dbuf);
+		lp->dbuf = None;
+	}
+	if (!lp->wireframe) {
+		if ((lp->dbuf = XCreatePixmap(display, MI_WINDOW(mi),
+			lp->width, lp->height, MI_DEPTH(mi))) == None) {
+			free_life3d(display, lp);
+			return;
+		}
+	}
+
 	DrawScreen(mi);
 }
 
@@ -2220,11 +1289,12 @@ draw_life3d(ModeInfo * mi)
 			End3D(lp);
 		return;
 	}
-	DrawScreen(mi);
+	if (lp->visible) /* kill static life also */
+		DrawScreen(mi);
 
 	MI_IS_DRAWN(mi) = True;
 
-	if (++lp->generation > MI_CYCLES(mi) || !lp->visible) {
+	if (++lp->generation > MI_CYCLES(mi) || !lp->visible || lp->noChangeCount >= 8) {
 		/*CountCells3D(lp) == 0) */
 		init_life3d(mi);
 	} else
@@ -2249,12 +1319,8 @@ release_life3d(ModeInfo * mi)
 	if (life3ds != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			life3dstruct *lp = &life3ds[screen];
-
-			if (lp->eraserhead.next != NULL)
-				End3D(lp);
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_life3d(MI_DISPLAY(mi), &life3ds[screen]);
 		free(life3ds);
 		life3ds = (life3dstruct *) NULL;
 	}
