@@ -484,7 +484,7 @@ static char mouse_modes[][MAXMODECHARS] =
 #ifdef MODE_swarm
 	"swarm",
 #endif
-#ifdef MODE_swarm
+#ifdef MODE_t3d
 	"t3d"
 #endif
 };
@@ -707,6 +707,8 @@ pickMode(void)
 	int         mode, i;
 
 	if (mode_indexes == NULL) {
+		if (!nmodes)
+			return 0;
 		if ((mode_indexes = (int *) calloc(nmodes,
 				sizeof (int))) == NULL) {
 			if (sequential)
