@@ -382,7 +382,7 @@ initialise_colours(COLOUR * colours, float saturate)
 static void
 set_black_and_white(SWIRL_P swirl, XColor * values)
 {
-	const int maxSize = 67; /* Saves DirectColor from crashing */
+	const unsigned long maxSize = 67; /* Saves DirectColor from crashing */
 	unsigned long white, black;
 
 	/* where is black and white? */
