@@ -508,7 +508,7 @@ leadingEmptyHeight(Polyominoes * polyo)
 static Bool
 readPolyominoes(void)
 {
-  int   size, n, index = 0, sum = 0;
+  int   size, n, polyindex = 0, sum = 0;
   int   counter, start_counter;
   int   polyomino, kinds, i, j, bonustype;
 
@@ -516,12 +516,12 @@ readPolyominoes(void)
     counter = 0, start_counter = 0;
     polytris[bonustype].mode.number = start_ominoes[bonustype];
     for (kinds = 0; kinds < area_kinds[bonustype]; kinds++) {
-      size = ominos[index++];
+      size = ominos[polyindex++];
       if (size > omino_squares[bonustype]) {
         (void) printf("tetris corruption: size %d\n", size);
         return False;
       }
-      n = ominos[index++];
+      n = ominos[polyindex++];
       if (sum > max_ominoes[bonustype]) {
         (void) printf("tetris corruption: n %d\n", n);
         return False;
@@ -534,18 +534,18 @@ readPolyominoes(void)
             bonustype, polyomino, counter);
           return False;
         }
-        if (polyomino != ominos[index]) {
-          (void) printf("tetris corruption: polyomino %d, ominos[index] %d\n",
-            polyomino, ominos[index]);
+        if (polyomino != ominos[polyindex]) {
+          (void) printf("tetris corruption: polyomino %d, ominos[polyindex] %d\n",
+            polyomino, ominos[polyindex]);
           return False;
         }
-        index++;  /* This is only there to "read" it */
+        polyindex++;  /* This is only there to "read" it */
         polytris[bonustype].polyomino[sum].rotation =
-          ominos[index++] + counter;
+          ominos[polyindex++] + counter;
         polytris[bonustype].polyomino[sum].reflection =
-          ominos[index++] + counter; /* not used */
+          ominos[polyindex++] + counter; /* not used */
 
-        if (ominos[index++]) {
+        if (ominos[polyindex++]) {
           if (start_counter < start_ominoes[bonustype]) {
             polytris[bonustype].mode.start[start_counter++] = sum;
           } else {
@@ -563,7 +563,7 @@ readPolyominoes(void)
         for (j = 0; j < size; j++) {
           for (i = 0; i < size; i++) {
             polytris[bonustype].polyomino[sum].shape[j * size + i] =
-        ominos[index++];
+        ominos[polyindex++];
           }
         }
         leadingEmptyWidth(&(polytris[bonustype].polyomino[sum]));
@@ -1055,8 +1055,6 @@ drawPolyomino(ModeInfo * mi)
         if (polytris[(int) tp->bonusNow].polyomino
                     [tp->curPolyomino.polyomino_number].shape
                     [j * tp->curPolyomino.size + i]) {
-          int xi, yj;
-
           xi = (tp->curPolyomino.xpos + i) % WELL_PERIMETER;
           yj = tp->curPolyomino.ypos + j;
           if (yj >= WELL_DEPTH) {
